package checkers;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import common.Board;
import common.Location;
import common.Move;
import common.Piece;
import common.Player;

/**
 * A game of checkers
 * 
 * The main method to call is move(). The available moves may be obtained via
 * getAvailableMoves() or the return value from move()
 */
public class Game {

	/**
	 * The current player (player that moves)
	 */
	Player current;
	/**
	 * The board
	 */
	Board board;
	/**
	 * The piece in a series of captures (or null)
	 */
	Piece inCapture;
	/**
	 * The available moves
	 */
	Map<Piece, Set<Move>> availableMoves = new HashMap<>();

	public Game() {
		setUpBoard();
		current = Player.WHITE; // White always moves first
		updateAvailableMoves();
		inCapture = null;
	}

	/**
	 * Set the pieces in their starting location
	 */
	private void setUpBoard() {
		board = new Board();

		for (int row = 0; row < 3; ++row)
			for (int column = row % 2; column < Board.COLUMNS; column += 2)
				board.add(new Pawn(Player.WHITE, new Location(row, column)));

		for (int row = 5; row < 8; ++row)
			for (int column = row % 2; column < Board.COLUMNS; column += 2)
				board.add(new Pawn(Player.BLACK, new Location(row, column)));
	}

	/**
	 * Performs a single move and return the available moves after the move
	 * completes.
	 * 
	 * If the move is a capturing move, the piece is queries with
	 * getNextCapture() to determine whether or not more captures are available.
	 * If there are more captures, the current player continues to capture with
	 * the same piece.
	 * 
	 * If the move is not capturing, or no more captures are available, the turn
	 * is moved to the other player and the list of available moves is
	 * constructed using updateAvailableMoves
	 * 
	 * @param move
	 *            The move to perform. Must be one of the available moves.
	 * @return The list of available moves after this move completes
	 */
	public Map<Piece, Set<Move>> move(Move move) {
		Piece piece = move.getPiece();

		if (piece.getPlayer() != current)
			throw new IllegalArgumentException("Moving piece must belong to the current player");

		if (!availableMoves.get(move.getPiece()).contains(move))
			throw new IllegalArgumentException("Current move is illegal");

		board.remove(piece);
		piece.getLocation().moveTo(move.getTarget());

		if (move.getTarget().getRow() == piece.getPlayer().getKingsRow()
				&& !(piece instanceof King)) {
			Piece king = new King(piece.getPlayer(), move.getTarget());
			piece = king;
		}

		board.add(piece);
		if (move.isCapture()) {
			board.remove(move.getCapture());

			Set<Move> captures = piece.getNextCapture(board);
			if (captures.size() > 0) {
				availableMoves.clear();
				availableMoves.put(piece, captures);
				return availableMoves;
			}
		}

		inCapture = null;

		current = current == Player.WHITE ? Player.BLACK : Player.WHITE;

		updateAvailableMoves();

		return availableMoves;
	}

	/**
	 * Collects and stores all the available moves for the current player
	 */
	private void updateAvailableMoves() {
		availableMoves.clear();
		for (Piece p : board) {
			if (p.getPlayer() == current) {
				Set<Move> moves = p.getPossibleMoves(board);
				if (moves.size() > 0)
					availableMoves.put(p, moves);
			}
		}
	}

	/**
	 * 
	 * @return the moves available to the current player, grouped by the moving
	 *         piece
	 */
	public Map<Piece, Set<Move>> getAvailableMoves() {
		return availableMoves;
	}

	public Board getBoard() {
		return board;
	}

	public Player getCurrentPlayer() {
		return current;
	}

	public void addPromotionListener(PiecePromotedListener listener) {
		// TODO: implement this method
	}

	public void removePromotionListener(PiecePromotedListener listener) {
		// TODO: implement this method
	}

	public void addMovementListener(PieceMovedListener listener) {
		// TODO: implement this method
	}

	public void removeMovementListener(PieceMovedListener listener) {
		// TODO: implement this method
	}

	public void addCaptureListener(PieceCapturedListener listener) {
		// TODO: implement this method
	}

	public void removeCaptureListener(PieceCapturedListener listener) {
		// TODO: implement this method
	}
}
