package common;

import java.util.AbstractCollection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Represent the state of the board game.
 * 
 */
public class Board extends AbstractCollection<Piece> {

	public static final int COLUMNS = 8;
	public static final int ROWS = 8;

	Piece[][] board = new Piece[ROWS][COLUMNS];

	/**
	 * 
	 * @param l
	 *            location
	 * @return the piece at location l or null if the location is empty
	 */
	public Piece getAt(Location l) {
		return getAt(l.getRow(), l.getColumn());
	}

	/**
	 * 
	 * @param row
	 *            the row coordinate
	 * @param column
	 *            the column coordinate
	 * @return the piece at the give coordinates
	 */
	public Piece getAt(int row, int column) {
		return board[row][column];
	}

	/**
	 * 
	 * @param location
	 *            location
	 * @return true iff the location is within the board dimensions.
	 */
	public boolean isValidLocation(Location location) {
		int row = location.getRow();
		int column = location.getColumn();

		if (row >= ROWS || row < 0 || column >= COLUMNS || column < 0)
			return false;

		return true;
	}

	/**
	 * 
	 * @param location location
	 * @return true iff the location is valid and empty.
	 */
	public boolean isEmptyLocation(Location location) {
		if (!isValidLocation(location))
			throw new IllegalArgumentException("Invalid location");

		int row = location.getRow();
		int column = location.getColumn();

		return board[row][column] == null;
	}

	/**
	 * Adds a piece to the board. 
	 * The piece is added only if its location is valid and empty.
	 */
	@Override
	public boolean add(Piece piece) {
		Location location = piece.getLocation();

		if (!isEmptyLocation(location))
			return false;

		board[location.getRow()][location.getColumn()] = piece;
		return true;
	}

	/**
	 * Removes a piece from the board
	 */
	@Override
	public boolean remove(Object o) {
		if (!(o instanceof Piece))
			return false;

		Piece p = (Piece) o;
		Location location = p.getLocation();
		if (getAt(location) != p)
			return false;

		board[location.getRow()][location.getColumn()] = null;
		return true;
	}

	@Override
	public Iterator<Piece> iterator() {
		List<Piece> pieces = new ArrayList<>();

		for (Piece[] row : board)
			for (Piece piece : row)
				if (piece != null)
					pieces.add(piece);

		return pieces.iterator();
	}

	@Override
	public int size() {
		int count = 0;

		for (Piece[] row : board)
			for (Piece piece : row)
				if (piece != null)
					++count;

		return count;
	}

}
