package common;

/**
 * Representation of a single move in the game. In a series of captures, each is
 * considered a separate move.
 * 
 */
public class Move {
	Location target;
	Piece piece;
	Piece capture;

	/**
	 * Initialize a Move describing movement without capture
	 * 
	 * @param p
	 *            the piece that moves
	 * @param target
	 *            the target location of the piece
	 */
	public Move(Piece p, Location target) {
		this(p, target, null);
	}

	/**
	 * Initialize a Move describing a capture
	 * 
	 * @param p
	 *            the piece that moves
	 * @param target
	 *            the target location of the piece
	 * @param capture
	 *            the piece being captured
	 */
	public Move(Piece p, Location target, Piece capture) {
		this.piece = p;
		this.target = target;
		this.capture = capture;
	}

	/**
	 * @return the piece making the move
	 */
	public Piece getPiece() {
		return piece;
	}

	/**
	 * @return the target location of the piece
	 */
	public Location getTarget() {
		return target;
	}

	/**
	 * @return true iff the current move is capturing another piece
	 */
	public boolean isCapture() {
		return capture != null;
	}

	/**
	 * @return the piece captured by this move, if the move is a capture.
	 */
	public Piece getCapture() {
		return capture;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((capture == null) ? 0 : capture.hashCode());
		result = prime * result + ((piece == null) ? 0 : piece.hashCode());
		result = prime * result + ((target == null) ? 0 : target.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Move other = (Move) obj;
		if (capture == null) {
			if (other.capture != null)
				return false;
		} else if (!capture.equals(other.capture))
			return false;
		if (piece == null) {
			if (other.piece != null)
				return false;
		} else if (!piece.equals(other.piece))
			return false;
		if (target == null) {
			if (other.target != null)
				return false;
		} else if (!target.equals(other.target))
			return false;
		return true;
	}

}
