package ui;

import java.awt.GridLayout;
import java.awt.Image;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.nio.file.Paths;
import java.util.Collection;

import javax.imageio.ImageIO;
import javax.swing.JPanel;

import checkers.Game;
import checkers.King;
import checkers.PieceCapturedEvent;
import checkers.PieceCapturedListener;
import checkers.PieceMovedEvent;
import checkers.PieceMovedListener;
import checkers.PiecePromotedEvent;
import checkers.PiecePromotedListener;

import common.Location;
import common.Move;
import common.Piece;
import common.Player;

/**
 * A UI element for the board.
 * 
 * This class shows that graphic representation of the board to the user and
 * produces events on location selection (in terms of board location) instead of
 * primitive mouse clicks
 * 
 */
public class BoardPanel extends JPanel {

	private static final long serialVersionUID = 4681820248634203902L;
	/**
	 * The square objects that are contained within the board (these are the UI
	 * elements that are drawn on this board)
	 */
	private Square[][] squares = new Square[common.Board.ROWS][common.Board.COLUMNS];

	// Graphic images
	private Image white_pawn;
	private Image white_king;
	private Image black_pawn;
	private Image black_king;
	private Image overlay;

	public BoardPanel() {
		setLayout(new GridLayout(8, 8, 0, 0));

		loadImages();

		for (int r = 0; r < common.Board.ROWS; ++r)
			for (int c = 0; c < common.Board.COLUMNS; ++c) {
				Square thisSquare = new Square(common.Board.ROWS - r - 1, c);
				thisSquare.addMouseListener(new MouseAdapter() {
					@Override
					public void mouseReleased(MouseEvent e) {
						// TODO: implement this method
					}
				});
				add(thisSquare);

				squares[common.Board.ROWS - r - 1][c] = thisSquare;
			}
	}

	/**
	 * Sets the game object this board should represent. This object will listen
	 * on the events from the game and update the UI accordingly.
	 * 
	 * @param game
	 *            the game to follow
	 */
	public void setGame(Game game) {
		for (int r = 0; r < common.Board.ROWS; ++r)
			for (int c = 0; c < common.Board.COLUMNS; ++c) {
				Piece piece = game.getBoard().getAt(r, c);
				squares[r][c].setImage(getImageForPiece(piece));
			}

		game.addCaptureListener(new PieceCapturedListener() {
			@Override
			public void pieceCaptured(Game sender, PieceCapturedEvent event) {
				Location l = event.getPiece().getLocation();
				squares[l.getRow()][l.getColumn()].setImage(null);
			}
		});

		game.addMovementListener(new PieceMovedListener() {
			@Override
			public void pieceMoved(Game game, PieceMovedEvent event) {
				Location source = event.getSourceLocation();
				Location l = event.getPiece().getLocation();
				squares[l.getRow()][l.getColumn()].setImage(squares[source
						.getRow()][source.getColumn()].getImage());
				squares[source.getRow()][source.getColumn()].setImage(null);
			}
		});

		game.addPromotionListener(new PiecePromotedListener() {
			@Override
			public void piecePromoted(Game source, PiecePromotedEvent event) {
				Location l = event.getPawn().getLocation();
				Location k = event.getKing().getLocation();
				squares[l.getRow()][l.getColumn()].setImage(null);
				squares[k.getRow()][l.getColumn()].setImage(event.getKing()
						.getPlayer() == Player.WHITE ? white_king : black_king);
			}
		});

		highlightPieces(null);
		highlightMoves(null);
		highlightSelected(null);
	}

	/**
	 * Highlights the squares of the pieces in the collection or remove all
	 * highlights if the pieces collection is empty or null
	 * 
	 * When highlighting pieces all move highlights are removed
	 * 
	 * @param pieces
	 *            a collection of pieces whose locations should be highlighted
	 */
	public void highlightPieces(Collection<Piece> pieces) {
		for (int r = 0; r < common.Board.ROWS; ++r)
			for (int c = 0; c < common.Board.COLUMNS; ++c) {
				squares[r][c].setHighlight(false);
			}

		if (pieces != null)
			for (Piece p : pieces)
				squares[p.getLocation().getRow()][p.getLocation().getColumn()]
						.setHighlight(true);

		highlightMoves(null);
	}

	/**
	 * Highlights the target squares of all the moves in the collection or turn
	 * of highlighting if the collection is empty or null
	 * 
	 * @param moves
	 *            the collection of moves
	 */
	public void highlightMoves(Collection<Move> moves) {
		for (int r = 0; r < common.Board.ROWS; ++r)
			for (int c = 0; c < common.Board.COLUMNS; ++c) {
				squares[r][c].setOverlay(null);
			}

		if (moves != null)
			for (Move m : moves)
				squares[m.getTarget().getRow()][m.getTarget().getColumn()]
						.setOverlay(overlay);

	}

	private Image getImageForPiece(Piece piece) {
		if (piece == null)
			return null;

		if (piece.getPlayer() == Player.WHITE)
			return (piece instanceof King) ? white_king : white_pawn;

		return (piece instanceof King) ? black_king : black_pawn;
	}

	private void loadImages() {
		try {
			white_pawn = ImageIO.read(Paths.get("resources/white_pawn.png")
					.toFile());
		} catch (IOException e) {
			e.printStackTrace();
		}

		try {
			white_king = ImageIO.read(Paths.get("resources/white_king.png")
					.toFile());
		} catch (IOException e) {
			e.printStackTrace();
		}

		try {
			black_pawn = ImageIO.read(Paths.get("resources/black_pawn.png")
					.toFile());
		} catch (IOException e) {
			e.printStackTrace();
		}

		try {
			black_king = ImageIO.read(Paths.get("resources/black_king.png")
					.toFile());
		} catch (IOException e) {
			e.printStackTrace();
		}

		try {
			overlay = ImageIO.read(Paths.get("resources/overlay.png").toFile());
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	public void addLocationListener(LocationSelectionListener listener) {
		// TODO: implement this method
	}

	public void removeLocationListener(LocationSelectionListener listener) {
		// TODO: implement this method
	}

	/**
	 * Marks the location of the input piece as selected or removes all
	 * selection from the board if the piece is null
	 * 
	 * @param selectedPiece the location to mark as selected
	 */
	public void highlightSelected(Piece selectedPiece) {
		for (int r = 0; r < common.Board.ROWS; ++r)
			for (int c = 0; c < common.Board.COLUMNS; ++c) {
				squares[r][c].setSelected(false);
			}
		if (selectedPiece != null)
			squares[selectedPiece.getLocation().getRow()][selectedPiece
					.getLocation().getColumn()].setSelected(true);
		repaint();
	}
}
