package ui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.border.Border;

import common.Location;

/**
 * A UI element of a single square on the board. The square may be drawn with a
 * border for highlighting and images for representation of pawns or available
 * moves
 */
public class Square extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 6625814798739057324L;
	private Location boardLocation;
	private Image image;
	private Image overlay;

	private boolean highlighted = false;
	private boolean selected = false;

	private static final Border HIGHLIGHT = BorderFactory.createLineBorder(
			Color.GREEN, 3);
	private static final Border SELECTED = BorderFactory.createLineBorder(
			Color.RED, 3);
	private static final Border EMPTY = BorderFactory.createEmptyBorder();

	/**
	 * Create the panel.
	 */
	public Square(int row, int column) {
		this.boardLocation = new Location(row, column);
		int square = (row % 2) ^ (column % 2);
		setBackground(square == 0 ? Color.BLACK : Color.WHITE);
	}

	/**
	 * @return the location of this square on the board
	 */
	public Location getBoardLocation() {
		return boardLocation;
	}

	/**
	 * Sets an image to display on this square. The image represent the piece in
	 * the current sqaure.
	 * 
	 * @param image
	 *            the image to be displayed
	 */
	public void setImage(Image image) {
		this.image = image;
		repaint();
	}

	@Override
	protected void paintComponent(Graphics go) {
		if (highlighted && !selected)
			setBorder(HIGHLIGHT);
		else if (selected)
			setBorder(SELECTED);
		else
			setBorder(EMPTY);
		super.paintComponent(go);

		Graphics2D g = (Graphics2D) go;
		g.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
				RenderingHints.VALUE_INTERPOLATION_BILINEAR);
		g.setRenderingHint(RenderingHints.KEY_RENDERING,
				RenderingHints.VALUE_RENDER_QUALITY);
		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		if (image != null)
			g.drawImage(image, 0, 0, getWidth(), getHeight(), null);

		if (overlay != null)
			g.drawImage(overlay, 0, 0, getWidth(), getHeight(), null);
	}

	/**
	 * Controls whether this square is highlighted. Highlighted squares has
	 * green border.
	 * 
	 * @param highlighted
	 *            true iff this square should be highlighted
	 */
	public void setHighlight(boolean highlighted) {
		this.highlighted = highlighted;
		repaint();
	}

	/**
	 * Sets an overlay image to be displayed in this square. The overlay marks
	 * available moves.
	 * 
	 * @param overlay
	 *            the image to overlay
	 */
	public void setOverlay(Image overlay) {
		this.overlay = overlay;
		repaint();
	}

	/**
	 * @return the image on the current square.
	 */
	public Image getImage() {
		return image;
	}

	public boolean isHighlighted() {
		return highlighted;
	}

	public boolean isOverlayed() {
		return overlay != null;
	}

	/**
	 * Controls whether this square is selected. Selected squares have red
	 * border.
	 * 
	 * @param b
	 *            true iff this square is selected
	 */
	public void setSelected(boolean b) {
		selected = b;
		repaint();
	}

}
